@@ -78,54 +78,6 @@ function show_message()
 	fi
 }
 
-function show_license_agreement()
-{
-	LICENSE="$STEAMROOT/steam_install_agreement.txt"
-	if [ ! -f "$STEAMCONFIG/steam_install_agreement.txt" ]; then
-		if [ ! -f "$LICENSE" ]; then
-			show_message --error $"Couldn't find Steam install license agreement, aborting!"
-			exit 1
-		fi
-
-		# See if they have been grandfathered in through the beta
-		SSAVersion=$(find -L "$STEAMCONFIG/`detect_package`" -name sharedconfig.vdf -exec fgrep SSAVersion {} \;)
-
-		if [ "$SSAVersion" != "" ]; then
-			answer=accepted
-		else
-			answer=declined
-			output=$(zenity --width 650 --height 500 --text-info --title=$"Steam Install Agreement" --filename="$LICENSE" --checkbox=$"I have read and accept these terms." 2>&1)
-			STATUS=$?
-			if echo $output | grep "status 1:" >/dev/null; then
-				# Zenity couldn't launch a window
-				STATUS=-1
-			fi
-			case $STATUS in
-			0)	# The agreement was accepted
-				answer=accepted
-				;;
-			1)	# The agreement was declined
-				;;
-			*)	# zenity wasn't available, try a fallback
-				tmpfile="$(mktemp || echo "/tmp/steam_message.txt")"
-				command="more \"$LICENSE\" || cat \"$LICENSE\"; echo -n $'Do you accept the terms of this agreement? [y/N]: '; read input; if [ x\$input = xy -o x\$input = xY ]; then echo accepted >\"$tmpfile\"; fi"
-				xterm -bg "#383635" -fg "#d1cfcd" -T $"Steam Install Agreement" -e "$command" || \
-					/bin/bash -c "$command"
-				if [ -f "$tmpfile" ]; then
-					read answer <"$tmpfile"
-					rm "$tmpfile"
-				fi
-				;;
-			esac
-			if [ "$answer" != "accepted" ]; then
-				exit 0
-			fi
-		fi
-
-		cp "$LICENSE" "$STEAMCONFIG"/
-	fi
-}
-
 
 function distro_description()
 {
@@ -265,6 +217,215 @@ function install_bootstrap()
 	return $STATUS
 }
 
+function pin_newer_runtime_libs ()
+{
+	# Set separator to newline just for this function
+	local IFS=$(echo -en "\n\b")
+	
+	# First argument is the runtime path
+	steam_runtime_path=`realpath $1`
+	
+	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
+	
+	# Associative array; indices are the SONAME, values are final path
+	declare -A host_libraries_32
+	declare -A host_libraries_64
+	
+	rm -rf "$steam_runtime_path/pinned_libs_32"
+	rm -rf "$steam_runtime_path/pinned_libs_64"
+	
+	# First, grab the list of system libraries from ldconfig and put them in the arrays
+	for ldconfig_output in `/sbin/ldconfig -XNv 2> /dev/null`
+	do
+		# If line starts with a leading / and contains :, it's a new path prefix
+		if [[ "$ldconfig_output" =~ ^/.*: ]]
+		then
+			library_path_prefix=`echo $ldconfig_output | cut -d: -f1`
+		else
+			# Otherwise it's a soname symlink -> library pair, build a full path to the soname link
+			leftside=${ldconfig_output% -> *}
+			soname=`echo $leftside | tr -d '[:space:]'`
+			soname_fullpath=$library_path_prefix/$soname
+			
+			# Left side better be a symlink
+			if [[ ! -L $soname_fullpath ]]; then continue; fi
+			
+			# Left-hand side of soname symlink should be *.so.%d
+			if [[ ! $soname_fullpath =~ .*\.so.[[:digit:]]+$ ]]; then continue; fi
+			
+			final_library=`readlink -f $soname_fullpath`
+			
+			# Target library must be named *.so.%d.%d.%d
+			if [[ ! $final_library =~ .*\.so.[[:digit:]]+.[[:digit:]]+.[[:digit:]]+$ ]]; then continue; fi
+			
+			# If it doesn't exist, skip as well
+			if [[ ! -f $final_library ]]; then continue; fi
+			
+			# Save into bitness-specific associative array with only SONAME as left-hand
+			if [[ `file -L $final_library` == *"32-bit"* ]]
+			then
+				host_libraries_32[$soname]=$soname_fullpath
+			elif [[ `file -L $final_library` == *"64-bit"* ]]
+			then
+				host_libraries_64[$soname]=$soname_fullpath
+			fi
+		fi
+	done
+	
+	mkdir "$steam_runtime_path/pinned_libs_32"
+	mkdir "$steam_runtime_path/pinned_libs_64"
+	
+	for find_output in `find "$steam_runtime_path" -type l | grep \\\.so`
+	do
+		# Left-hand side of soname symlink should be *.so.%d
+		if [[ ! $find_output =~ .*\.so.[[:digit:]]+$ ]]; then continue; fi
+		
+		soname_symlink=$find_output
+		
+		final_library=`readlink -f $soname_symlink`
+		
+		# Target library must be named *.so.%d.%d.%d
+		if [[ ! $final_library =~ .*\.so.([[:digit:]]+).([[:digit:]]+).([[:digit:]]+)$ ]]; then continue; fi
+		
+		# This pattern strips leading zeroes, which could otherwise cause bash to interpret the value as binary/octal below
+		r_lib_major=$((10#${BASH_REMATCH[1]}))
+		r_lib_minor=$((10#${BASH_REMATCH[2]}))
+		r_lib_third=$((10#${BASH_REMATCH[3]}))
+		
+		# If it doesn't exist, skip as well
+		if [[ ! -f $final_library ]]; then continue; fi
+		
+		host_library=""
+		host_soname_symlink=""
+		bitness="unknown"
+		
+		soname=$(basename "$soname_symlink")
+		
+		# If we had entries in our arrays, get them
+		if [[ `file -L $final_library` == *"32-bit"* ]]
+		then
+			if [ ! -z ${host_libraries_32[$soname]+isset} ]
+			then
+				host_soname_symlink=${host_libraries_32[$soname]}
+			fi
+			bitness="32"
+		elif [[ `file -L $final_library` == *"64-bit"* ]]
+		then
+			if [ ! -z ${host_libraries_64[$soname]+isset} ]
+			then
+				host_soname_symlink=${host_libraries_64[$soname]}
+			fi
+			bitness="64"
+		fi
+		
+		# Do we have a host library found for the same SONAME?
+		if [[ ! -f $host_soname_symlink || $bitness == "unknown" ]]; then continue; fi
+		
+		host_library=`readlink -f $host_soname_symlink`
+		
+		if [[ ! -f $host_library ]]; then continue; fi
+		
+		#echo $soname ${host_libraries[$soname]} $r_lib_major $r_lib_minor $r_lib_third
+
+		# Pretty sure the host library already matches, but we need the rematch anyway
+		if [[ ! $host_library =~ .*\.so.([[:digit:]]+).([[:digit:]]+).([[:digit:]]+)$ ]]; then continue; fi
+		
+		h_lib_major=$((10#${BASH_REMATCH[1]}))
+		h_lib_minor=$((10#${BASH_REMATCH[2]}))
+		h_lib_third=$((10#${BASH_REMATCH[3]}))
+		
+		runtime_version_newer="no"
+		
+		if [[ $h_lib_major -lt $r_lib_major ]]; then
+			runtime_version_newer="yes"
+		fi
+		
+		if [[ $h_lib_major -eq $r_lib_major && $h_lib_minor -lt $r_lib_minor ]]; then
+			runtime_version_newer="yes"
+		fi
+		
+		if [[ $h_lib_major -eq $r_lib_major && $h_lib_minor -eq $r_lib_minor && $h_lib_third -lt $r_lib_third ]]; then
+			runtime_version_newer="yes"
+		fi
+		
+		# There's a set of libraries that have to work together to yield a working dock
+		# We're reasonably convinced our set works well, and only pinning a handful would
+		# induce a mismatch and break the dock, so always pin all of these for Steam (32-bit)
+		if [[ $bitness == "32" ]]
+		then
+			if [[ 	"$soname" == "libgtk-x11-2.0.so.0"  || \
+					"$soname" == "libdbusmenu-gtk.so.4"  || \
+					"$soname" == "libdbusmenu-glib.so.4" || \
+					"$soname" == "libdbus-1.so.3" ]]
+			then
+				runtime_version_newer="yes"
+			fi
+		fi
+		
+		
+		if [[ $runtime_version_newer == "yes" ]]; then
+			echo Found newer runtime version for $bitness-bit $soname. Host: $h_lib_major.$h_lib_minor.$h_lib_third Runtime: $r_lib_major.$r_lib_minor.$r_lib_third 
+			ln -s "$final_library" "$steam_runtime_path/pinned_libs_$bitness/$soname"
+			# Keep track of the exact version name we saw on the system at pinning time to check later
+			echo "$host_soname_symlink" > "$steam_runtime_path/pinned_libs_$bitness/system_$soname"
+			echo "$host_library" >> "$steam_runtime_path/pinned_libs_$bitness/system_$soname"
+			touch "$steam_runtime_path/pinned_libs_$bitness/has_pins"
+		fi
+	done
+}
+
+function check_pins ()
+{
+	# Set separator to newline just for this function
+	local IFS=$(echo -en "\n\b")
+	
+	# First argument is the runtime path
+	steam_runtime_path=`realpath $1`
+	
+	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
+	
+	pins_need_redoing="no"
+	
+	# If we had the runtime previously unpacked but never ran the pin code, do it now
+	if [[ ! -d "$steam_runtime_path/pinned_libs_32" ]]
+	then
+		pins_need_redoing="yes"
+	fi
+	
+	if [[ -f "$steam_runtime_path/pinned_libs_32/has_pins" || -f "$steam_runtime_path/pinned_libs_64/has_pins" ]]
+	then
+		for pin in "$steam_runtime_path"/pinned_libs_*/system_*
+		do
+			host_sonamesymlink=`head -1 "$pin"`
+			host_library=`tail -1 "$pin"`
+			
+			# Follow the host SONAME symlink we saved in the first line of the pin entry
+			host_actual_library=`readlink -f $host_sonamesymlink`
+			
+			# It might not exist anymore if it got uninstalled or upgraded to a different major version
+			if [[ ! -f $host_actual_library ]]
+			then
+				pins_need_redoing="yes"
+			fi
+			
+			# We should end up at the same lib we saved in the second line
+			if [[ $host_actual_library != $host_library  ]]
+			then
+				# Mismatch, it could have gotten upgraded
+				pins_need_redoing="yes"
+			fi
+		done
+	fi
+	
+	if [[ $pins_need_redoing == "yes" ]]
+	then
+		echo Pins potentially out-of-date, rebuilding...
+		pin_newer_runtime_libs "$steam_runtime_path"
+	else
+		echo Pins up-to-date!
+	fi
+}
+
 function runtime_supported()
 {
 	case "$(detect_distro)-$(detect_release)" in
@@ -329,6 +490,7 @@ function unpack_runtime()
 	if ! has_runtime_archive; then
 		if [ -d "$STEAM_RUNTIME" ]; then
 			# The runtime is unpacked, let's use it!
+			check_pins "$STEAM_RUNTIME"
 			return 0
 		fi
 		return 1
@@ -336,6 +498,7 @@ function unpack_runtime()
 
 	# Make sure we haven't already unpacked them
 	if [ -f "$STEAM_RUNTIME/checksum" ] && cmp "$STEAM_RUNTIME.checksum" "$STEAM_RUNTIME/checksum" >/dev/null; then
+		check_pins "$STEAM_RUNTIME"
 		return 0
 	fi
 
@@ -368,6 +531,8 @@ function unpack_runtime()
 	if ! cp "$STEAM_RUNTIME.checksum" "$STEAM_RUNTIME/checksum"; then
 		return 6
 	fi
+	# Unpacked a new runtime, pin any newer system libs with symlinks in a special dir
+	pin_newer_runtime_libs "$STEAM_RUNTIME"
 	return 0
 }
 
@@ -488,7 +653,6 @@ function steamos_arg()
 
 	return 1; # 1 == false in bash speak
 }
-
  
 #determine platform
 UNAME=`uname`
@@ -511,10 +675,10 @@ STEAMEXEPATH=$PLATFORM/$STEAMEXE
 
 # common variables for later
 
-# We use ~/.steam for bootstrap symlinks so that we can easily
+# We use ${HOME%/}/.steam for bootstrap symlinks so that we can easily
 # tell partners where to go to find the Steam libraries and data.
 # This is constant so that legacy applications can always find us in the future.
-STEAMCONFIG=~/.steam
+STEAMCONFIG="${HOME%/}/.steam" # Drop tailing slash in home folder if it exists.
 PIDFILE="$STEAMCONFIG/steam.pid" # pid of running steam for this user
 STEAMBIN32LINK="$STEAMCONFIG/bin32"
 STEAMBIN64LINK="$STEAMCONFIG/bin64"
@@ -543,9 +707,6 @@ if [ "${1-}" = "--reset" ]; then
 fi
 
 if [ "$INITIAL_LAUNCH" ]; then
-	# Show the license agreement, if needed
-	show_license_agreement
-
 	if [ -z "${STEAMSCRIPT:-}" ]; then
 		STEAMSCRIPT="/usr/bin/`detect_package`"
 	fi
@@ -574,7 +735,7 @@ if [ "$INITIAL_LAUNCH" ]; then
 	fi
 
 	# Temporary bandaid until everyone has the new libsteam_api.so
-	rm -f ~/.steampath && ln -s "$STEAMCONFIG/bin32/steam" ~/.steampath
+	rm -f ~/.steampath && ln -s "$STEAMCONFIG/sdk32/steam" ~/.steampath
 	rm -f ~/.steampid && ln -s "$PIDFILE" ~/.steampid
 	rm -f ~/.steam/bin && ln -s "$STEAMBIN32LINK" ~/.steam/bin
 	# Uncomment this line when you want to remove the bandaid
@@ -604,7 +765,7 @@ if [ "$STEAM_RUNTIME" = "debug" ]; then
 		if [ -z "${STEAM_RUNTIME_DEBUG-}" ]; then
 			STEAM_RUNTIME_DEBUG="$(cat "$STEAM_RUNTIME/version.txt" | sed 's,-release,-debug,')"
 		fi
-		if [ -z "{$STEAM_RUNTIME_DEBUG_DIR-}" ]; then
+		if [ -z "${STEAM_RUNTIME_DEBUG_DIR-}" ]; then
 			STEAM_RUNTIME_DEBUG_DIR="$STEAMROOT/$PLATFORM"
 		fi
 		if [ ! -d "$STEAM_RUNTIME_DEBUG_DIR/$STEAM_RUNTIME_DEBUG" ]; then
@@ -658,8 +819,10 @@ if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 				export PATH="$STEAM_RUNTIME/i386/bin:$STEAM_RUNTIME/i386/usr/bin:$PATH"
 				;;
 		esac
+		
+		export STEAM_RUNTIME_LIBRARY_PATH=`$STEAM_RUNTIME/run.sh --print-steam-runtime-library-paths`
 
-		export LD_LIBRARY_PATH="$STEAM_RUNTIME/i386/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/lib:$STEAM_RUNTIME/i386/usr/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/usr/lib:$STEAM_RUNTIME/amd64/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/lib:$STEAM_RUNTIME/amd64/usr/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/usr/lib:${LD_LIBRARY_PATH-}"
+		export LD_LIBRARY_PATH="$STEAM_RUNTIME_LIBRARY_PATH:${LD_LIBRARY_PATH-}"
 	else
 		echo "Unpack runtime failed, error code $?"
 		show_message --error $"Couldn't set up the Steam Runtime. Are you running low on disk space?\nContinuing..."
@@ -713,24 +876,10 @@ else
 fi
 STATUS=$?
 
-# Restore paths before unpacking the bootstrap if we need to.
+# Restore paths before restarting if we need to.
 export PATH="$SYSTEM_PATH"
 export LD_LIBRARY_PATH="$SYSTEM_LD_LIBRARY_PATH"
 
-if [ "$UNAME" = "Linux" ]; then
-	if [ "$INITIAL_LAUNCH" -a \
-	     $STATUS -ne $MAGIC_RESTART_EXITCODE -a \
-	     -f "$STEAMSTARTING" -a \
-	     -z "${STEAM_INSTALLED_BOOTSTRAP-}" ]; then
-		# Launching the bootstrap failed, try reinstalling
-		if reset_steam; then
-			# We were able to reinstall the bootstrap, try again
-			export STEAM_INSTALLED_BOOTSTRAP=1
-			STATUS=$MAGIC_RESTART_EXITCODE
-		fi
-	fi
-fi
-
 # If steam requested to restart, then restart
 if [ $STATUS -eq $MAGIC_RESTART_EXITCODE ] ; then
 	echo "Restarting Steam by request..."
